#include <iostream>
#include <vector>
using namespace std;
 
//Line A

#include "TTree.h"
#include "TFile.h"
#include "TLorentzVector.h"
#include "TVector3.h"
#include "TH1.h"
#include "TH2.h"
#include "TF1.h"

//Line B
//My include file                                                                                                                                                                 
#include "myTree.h"
//Line C
//Structure "Bucket Of Information"
struct myBOI_t {
  TLorentzVector mo;
  int pid;
  int sciDet;
};
//Line D

int main(int argc, char **argv) {
  cout<<"Hello World"<<endl;
  //Line M1
  //Define integer nToRun;                                                                                                                                                        
  int nToRun = 1;
  //Line M2
  //Define the histograms                                                                                                                                                         
  TH2D *hBetaVsP     = new TH2D("hBetaVsP","myTitle",60,0.0,6.0,110,0.0,1.1);
  TH2D *hBetaVsPPro  = new TH2D("hBetaVsPPro","myTitle",60,0.0,6.0,110,0.0,1.1);
  TH1D *hMassKpKm = new TH1D("hMassKpKm","KK Mass",100,0.9,1.5);
  //Line M3
  //Setup the output file                                                                                                                                                         
  TFile *outFile = new TFile("myOutFile.root","RECREATE");
  //Line M4

  //Setup the input file and tree                                                                                                                             
  TFile* inFile=new TFile("./skim11_005367.hipo.root");
  //Get tree out of the file
  TTree *inTree = (TTree*)inFile->Get("clas12");
  //Define the tree structure
  myTree_t myTree;
  //Get the tree branches into myTree structure
  getBranchesStage1(inTree,&myTree);
  //Find out how many events are in the tree and set to nToRun
  nToRun = inTree->GetEntries();
  //Print the number of events to process in the terminal
  cout<<"Number of events to process = "<<nToRun<<endl;

  //Line M5
  for(Int_t i=0;i<nToRun;i++){
    inTree->GetEntry(i);
    //Print every 50000 events                                                                                                                                                    
    if (i%50000 == 0) cout<<"Events processed = "<<i<<endl;
    //Find out how many particles are in the event                                                                                                                                
    int nREC_Particle = myTree.REC_Particle_pid->size();
    //Line L1                                                  
    //Setting up Bucket-Of-Information particle vectors 
    myBOI_t boi;
    vector <myBOI_t> proBOI;
    vector <myBOI_t> kpBOI;
    vector <myBOI_t> kmBOI;
    vector <myBOI_t> pipBOI;
    vector <myBOI_t> pimBOI;

    //Loop over each particle in the event                                                                                                                                        
    for(int index = 0; index<nREC_Particle; index++){
      int pid   = myTree.REC_Particle_pid->at(index);
      Float_t px = myTree.REC_Particle_px->at(index);
      double py = myTree.REC_Particle_py->at(index);
      double pz = myTree.REC_Particle_pz->at(index);
      double p  = sqrt(pow(px,2)+pow(py,2)+pow(pz,2));
      double beta   = myTree.REC_Particle_beta->at(index);
      //Line P1
      int partStat = myTree.REC_Particle_status->at(index);
      int partStatDet = partStat - partStat%1000;
      boi.sciDet = partStatDet;                      
                                                                                                                                             
      hBetaVsP->Fill(p,beta);
      if (pid == 2212) hBetaVsPPro->Fill(p,beta);
      //Line P2
      double eval;
      //PROTON
      if (pid == 2212) {
	eval = sqrt(pow(p,2)+pow(0.938,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	proBOI.push_back(boi);
      }
      //PI PLUS
      if (pid == 211) {
	eval = sqrt(pow(p,2)+pow(0.139570,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	pipBOI.push_back(boi);
      }
      //PI MINUS
      if (pid == -211) {
	eval = sqrt(pow(p,2)+pow(0.139570,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	pimBOI.push_back(boi);
      }
      //K PLUS (ONLY DEFINED FOR FORWARD TIME-OF-FLIGHT)
      if (pid == 321 && partStatDet == 2000) {
	eval = sqrt(pow(p,2)+pow(0.493677,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	kpBOI.push_back(boi);
      }
      //K MINUS (ONLY DEFINED FOR FORWARD TIME-OF-FLIGHT)
      if (pid == -321 && partStatDet == 2000) {
	eval = sqrt(pow(p,2)+pow(0.493677,2));
	boi.mo.SetPxPyPzE(px,py,pz,eval);
	kmBOI.push_back(boi);
      }
    }
    //Line L2
    int npro = proBOI.size();
    int nkp = kpBOI.size();
    int nkm = kmBOI.size();
    int npip = pipBOI.size();
    int npim = pimBOI.size();
    if (nkp == 1 && nkm == 1){
      TLorentzVector moKpKm = kpBOI[0].mo + kmBOI[0].mo;
      hMassKpKm->Fill(moKpKm.Mag());
    }
  }

  //Line M6
  outFile->cd();

  hBetaVsP->Write();
  hBetaVsPPro->Write();
  hMassKpKm->Write();
  outFile->Write();
  //Line M7
  return 0;
}




